<?php
namespace TYPO3\CMS\Reports\Tests\Unit\Report\Status;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Christian Kuhn <lolli@schwarzbu.ch>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Test case
 */
class Typo3StatusTest extends \TYPO3\CMS\Core\Tests\UnitTestCase {

	/**
	 * Set up
	 */
	public function setUp() {
		$GLOBALS['LANG'] = $this->getMock('TYPO3\\CMS\\Lang\\LanguageService', array(), array(), '', FALSE);
	}

	/**
	 * @test
	 */
	public function getStatusReturnsOldXclassStatusObjectWithSeverityOkIfNoOldXclassExists() {
		$GLOBALS['TYPO3_CONF_VARS']['BE']['XCLASS'] = array();
		$GLOBALS['TYPO3_CONF_VARS']['FE']['XCLASS'] = array();
		$fixture = new \TYPO3\CMS\Reports\Report\Status\Typo3Status;
		$result = $fixture->getStatus();
		$statusObject = $result['oldXclassStatus'];
		$this->assertSame(\TYPO3\CMS\Reports\Status::OK, $statusObject->getSeverity());
	}

	/**
	 * @test
	 */
	public function getStatusReturnsOldXclassStatusObjectWithSeverityNoticeIfOldXclassExists() {
		$GLOBALS['TYPO3_CONF_VARS']['BE']['XCLASS'] = array('foo' => 'bar');
		$fixture = new \TYPO3\CMS\Reports\Report\Status\Typo3Status;
		$result = $fixture->getStatus();
		$statusObject = $result['oldXclassStatus'];
		$this->assertSame(\TYPO3\CMS\Reports\Status::NOTICE, $statusObject->getSeverity());
	}

	/**
	 * @test
	 */
	public function getStatusReturnsXclassStatusObjectWithSeverityOkIfNoXclassExists() {
		$GLOBALS['TYPO3_CONF_VARS']['SYS']['Objects'] = array();
		$fixture = new \TYPO3\CMS\Reports\Report\Status\Typo3Status;
		$result = $fixture->getStatus();
		$statusObject = $result['registeredXclass'];
		$this->assertSame(\TYPO3\CMS\Reports\Status::OK, $statusObject->getSeverity());
	}

	/**
	 * @test
	 */
	public function getStatusReturnsXclassStatusObjectWithSeverityNoticeIfXclassExists() {
		$GLOBALS['TYPO3_CONF_VARS']['SYS']['Objects'] = array(
			'foo' => array(
				'className' => 'bar',
			)
		);
		$fixture = new \TYPO3\CMS\Reports\Report\Status\Typo3Status;
		$result = $fixture->getStatus();
		$statusObject = $result['registeredXclass'];
		$this->assertSame(\TYPO3\CMS\Reports\Status::NOTICE, $statusObject->getSeverity());
	}
}
